local ADDON, Addon = ...;

local TITLE_HEIGHT = 10;

local ItemSlot = Addon.ItemSlot;

local ItemPart = { };
ItemPart.__index = ItemPart;

Addon.ItemPart = ItemPart;

function ItemPart:New(dec, last, title, storeKey, enabled)
	local self = { ["Items"] = {} };
	local name = ADDON .. "ItemPart" .. dec
	
	self.id = dec
	setmetatable(self, ItemPart);
	self.Frame = CreateFrame("Frame", name, MerchandrixItemFrame, ItemPart);
	
	self.Frame:RegisterEvent("ADDON_LOADED");
	Addon.EventHandler:RegisterEvent(self, "BAG_EVENT_UPDATE");
	
	self.Frame:SetScript("OnEvent", function(f, event, ...)
		if self[event] then
			self[event](self, event, ...)
		end		
	end);
	
	self:UpdateSize();
	
	self.storeKey = storeKey;
	
	self.Frame:SetBackdrop({bgFile = "Interface/FrameGeneral/UI-Background-Marble", 
						edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
						tile = false, tileSize = 16, edgeSize = 16, 
						insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	
	self:CreateTitle(title);
	self:CreateCheckBox();
	
	self:SetEnabled(enabled);
	
	return self;
end

function ItemPart:ADDON_LOADED(event, ad)
	if ad ~= ADDON then return end
	self.Frame:UnregisterEvent(event);
	self:SetEnabled(EnabledParts[self.id]);
	
	for _, item in ipairs(self.Items) do
		item:UpdateState()
	end
end

function ItemPart:BAG_EVENT_UPDATE()
	self:Clear();
	local sortFunc = nil;
	
	if self.storeKey == "weapon" or self.storeKey == "armor" then
		sortFunc = sortedArmor;
	else
		sortFunc = sortedOther;
	end
	
	for idx, item in sortFunc(Addon.TypeStore[self.storeKey]) do
		self:AddItem(item, idx);
	end
	self:UpdateSize();
end

function ItemPart:CreateTitle(title)
	local frame = self.Frame;
	local font = frame:CreateFontString(frame:GetName() .. "Title");
	font:SetPoint("TOPLEFT", 3, -10);
	font:SetFontObject(GameFontNormal);
	font:SetText(title);
	font:SetSize(font:GetStringWidth() + 20, TITLE_HEIGHT);
end

function ItemPart:CheckBoxClick()
	if self:GetChecked() then
		self.part:SetEnabled(true)
	else
		self.part:SetEnabled(false)
	end
end

function ItemPart:CreateCheckBox()
	local frame = self.Frame
	local box = CreateFrame("CheckButton", "$parentCheckBox", frame, "InterfaceOptionsCheckButtonTemplate")
	local func = self.CheckBoxClick;
	
	self.CheckBox = box
	
	box.part = self;
	
	box:SetPoint("TOPRIGHT", -75, -3);
	box.Text:SetText(Addon.L["Aktivieren"]);
	box.tooltipText = Addon.L["Aktiviert_Verkauf"];
	
	box:SetScript("OnClick", func);
	
	return box
end

function ItemPart:UpdateSize()
	local numItems = self:NumShownItems();
	local buttonSize = MerchandrixConfig.general.buttonSize;
	local newHeight = math.ceil(numItems / MerchandrixConfig.general.numColumns) * (buttonSize + 5) + TITLE_HEIGHT + 30;
	local width = (buttonSize + 5) * MerchandrixConfig.general.numColumns + 15;
	
	if numItems == 0 then
		self.Frame:SetSize(0, 0);
		self.Frame:Hide();
	else
		self.Frame:Show();
	end
	
	self.Frame:SetSize(width, newHeight);
	MerchandrixItemFrame:UpdateSize();
end

--[[ ItemButton ]]--

function ItemPart:CreateSellButton(slot)
	local b = ItemSlot:New(slot, self);
	
	table.insert(self.Items, b);
	
	return b;
end

function ItemPart:PlaceSellButton(btn)
	local itemNum = self:GetItemIndex(btn);
	local slot = btn.slot;
	
	if not itemNum then return end

	btn:ClearAllPoints();
	
	local function PlaceRight()
		btn:SetPoint("LEFT", self:GetLastItem(btn.slot), "RIGHT", 5, 0);
	end
	local function PlaceBottom()
		local topItem = self:GetTopItem(slot);
		if not topItem then
			btn:SetPoint("TOPLEFT", self.Frame,  10, -20 - TITLE_HEIGHT);
		else
			btn:SetPoint("TOP", topItem, "BOTTOM", 0, -5);
		end
	end
	
	local params = { select(2, self:GetColumnNums()) };
	table.insert(params, 0);
	local mapping = { { PlaceRight, unpack(params) }, { PlaceBottom, 1 } };
	map(itemNum % MerchandrixConfig.general.numColumns) : to(mapping); -- Place Button
end

function ItemPart:Clear()
	for _, item in ipairs(self.Items) do
		item:Hide();
	end
end

function ItemPart:AddItem(slot, idx)
	local item = self.Items[idx];
	
	if item then
		item:Set(slot)
	else
		item = self:CreateSellButton(slot)
		self:PlaceSellButton(item);
	end
	
	item:Show();
	item:UpdateState();
end

function ItemPart:UseConfig()
	for _, item in ipairs(self.Items) do
		self:PlaceSellButton(item);
		item:UseConfig()
	end
	
	self:UpdateSize()
end

function ItemPart:GetItemIndex(btn)
	for i, item in ipairs(self.Items) do
		if item == btn then
			return i;
		end
	end
	return nil;
end

function ItemPart:GetButtonById(id)
	for _, item in ipairs(self.Items) do
		if item.id == id then
			return item
		end
	end
end

function ItemPart:GetItem(slot)
	for _, item in ipairs(self.Items) do
		if item.id == Addon:GetID(slot) and item:IsShown() then
			return item
		end
	end
	
	return nil
end

function ItemPart:GetTopItem(slot)
	local idx = self:GetIndexOfItem(slot);
	if not idx then return nil end
	idx = idx - MerchandrixConfig.general.numColumns;
	return self.Items[idx];
end

function ItemPart:GetIndexOfItem(slot)
	for i, p in ipairs(self.Items) do
		if p.id == Addon:GetID(slot) then
			return i;
		end
	end
	return nil;
end

function ItemPart:GetLastItem(slot)
	for i, it in ipairs(self.Items) do
		if it.id == Addon:GetID(slot) then
			return self.Items[i - 1];
		end
	end
end

function ItemPart:GetColumnNums()
	local idx = {};
	for i = 1, MerchandrixConfig.general.numColumns do
		idx[i] = i;
	end
	return unpack(idx);
end

function ItemPart:NumShownItems()
	local counter = 0;
	for _, item in ipairs(self.Items) do
		if item:IsShown() then
			counter = counter + 1
		end
	end
	return counter;
end

function ItemPart:GetHiddenButton()
	for _, btn in ipairs(self.Items) do
		if not btn:IsShown() then
			return btn
		end
	end
	return nil
end

function ItemPart:RemoveItemBetween(id)
	local found = false;
	for i, item in ipairs(self.Items) do
		if item.id == id then
			found = true
		end
		
		if found and self.Items[i + 1] then
			item:Set(self.Items[i + 1].slot);
		end
	end
	
	if found then
		local lastNotHidden = self:GetLastNotHiddenItem();
		if lastNotHidden then
			lastNotHidden:Hide();
		end
	end
	
	self:UpdateSize();
end

function ItemPart:GetHiddenItems()
	local hidden = {};
	for _, item in ipairs(self.Items) do
		if not item:IsShown() then
			table.insert(hidden, item);
		end
	end
	return hidden;
end

function ItemPart:GetLastNotHiddenItem()
	return self.Items[self:NumShownItems()];
end

function ItemPart:GetEnabledItems()
	if not self:GetEnabled() then return {} end
	
	local items = {}
	for _, item in ipairs(self.Items) do
		if item:GetEnabled() and item:IsShown() then
			table.insert(items, item.slot)
		end
	end
	
	return items
end

function ItemPart:SetEnabled(enabled)
	self.CheckBox:SetChecked(enabled)
	EnabledParts[self.id] = enabled;
	
	for _, item in ipairs(self.Items) do
		item:UpdateState();
	end
end

function ItemPart:GetEnabled()
	return EnabledParts[self.id];
end